# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Messages.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import CommonTypes_pb2 as CommonTypes__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='Messages.proto',
  package='naothmessages',
  syntax='proto2',
  serialized_pb=_b('\n\x0eMessages.proto\x12\rnaothmessages\x1a\x11\x43ommonTypes.proto\"%\n\x06\x43MDArg\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\r\n\x05\x62ytes\x18\x03 \x01(\x0c\"8\n\x03\x43MD\x12\x0c\n\x04name\x18\x01 \x02(\t\x12#\n\x04\x61rgs\x18\x02 \x03(\x0b\x32\x15.naothmessages.CMDArg\"+\n\rStopwatchItem\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x0c\n\x04time\x18\x02 \x02(\r\"@\n\x0bStopwatches\x12\x31\n\x0bstopwatches\x18\x01 \x03(\x0b\x32\x1c.naothmessages.StopwatchItem\"\xb2\x01\n\x08PlotItem\x12.\n\x04type\x18\x01 \x02(\x0e\x32 .naothmessages.PlotItem.PlotType\x12\x0c\n\x04name\x18\x02 \x02(\t\x12\r\n\x05value\x18\x03 \x01(\x01\x12\t\n\x01x\x18\x04 \x01(\x01\x12\t\n\x01y\x18\x05 \x01(\x01\x12\x10\n\x08rotation\x18\x06 \x01(\x01\"1\n\x08PlotType\x12\x0b\n\x07\x44\x65\x66\x61ult\x10\x01\x12\n\n\x06Plot2D\x10\x02\x12\x0c\n\x08Origin2D\x10\x03\"J\n\x0cPlotStroke2D\x12\x0c\n\x04name\x18\x02 \x02(\t\x12,\n\x06points\x18\x01 \x03(\x0b\x32\x1c.naothmessages.DoubleVector2\"a\n\x05Plots\x12&\n\x05plots\x18\x01 \x03(\x0b\x32\x17.naothmessages.PlotItem\x12\x30\n\x0bplotstrokes\x18\x02 \x03(\x0b\x32\x1b.naothmessages.PlotStroke2D\"\xf8\x01\n\x0bXABSLSymbol\x12\x0e\n\x02id\x18\x01 \x02(\x05:\x02-1\x12<\n\x04type\x18\x02 \x02(\x0e\x32%.naothmessages.XABSLSymbol.SymbolType:\x07Unknown\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x14\n\x0c\x64\x65\x63imalValue\x18\x0b \x01(\x01\x12\x11\n\tboolValue\x18\x0c \x01(\x08\x12\x11\n\tenumValue\x18\r \x01(\x05\x12\x12\n\nenumTypeId\x18\x0e \x01(\x05\"=\n\nSymbolType\x12\x0b\n\x07\x44\x65\x63imal\x10\x00\x12\x0b\n\x07\x42oolean\x10\x01\x12\x08\n\x04\x45num\x10\x02\x12\x0b\n\x07Unknown\x10\x03\"\xfd\x01\n\x17XABSLActiveOptionSparse\x12\x0e\n\x02id\x18\x01 \x02(\x05:\x02-1\x12\x1a\n\x0ftimeOfExecution\x18\x03 \x01(\x03:\x01\x30\x12\x13\n\x0b\x61\x63tiveState\x18\x04 \x01(\x05\x12\x11\n\tstateTime\x18\x05 \x01(\x03\x12\x19\n\x11\x64\x65\x63imalParameters\x18\x06 \x03(\x01\x12\x19\n\x11\x62ooleanParameters\x18\x07 \x03(\x08\x12\x1c\n\x14\x65numeratedParameters\x18\x08 \x03(\x05\x12:\n\x10\x61\x63tiveSubActions\x18\t \x03(\x0b\x32 .naothmessages.XABSLActionSparse\"\x8b\x02\n\x11XABSLActionSparse\x12\x42\n\x04type\x18\x02 \x02(\x0e\x32+.naothmessages.XABSLActionSparse.ActionType:\x07Unknown\x12*\n\x06symbol\x18\x03 \x01(\x0b\x32\x1a.naothmessages.XABSLSymbol\x12\x36\n\x06option\x18\x04 \x01(\x0b\x32&.naothmessages.XABSLActiveOptionSparse\"N\n\nActionType\x12\n\n\x06Option\x10\x00\x12\x11\n\rBasicBehavior\x10\x01\x12\x14\n\x10SymbolAssignment\x10\x02\x12\x0b\n\x07Unknown\x10\x03\"\x93\x03\n\x0fSymbolValueList\x12<\n\x07\x64\x65\x63imal\x18\x01 \x03(\x0b\x32+.naothmessages.SymbolValueList.DoubleSymbol\x12=\n\x07\x62oolean\x18\x02 \x03(\x0b\x32,.naothmessages.SymbolValueList.BooleanSymbol\x12=\n\nenumerated\x18\x03 \x03(\x0b\x32).naothmessages.SymbolValueList.EnumSymbol\x1a;\n\x0c\x44oubleSymbol\x12\x0e\n\x02id\x18\x01 \x02(\x05:\x02-1\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\x01\x1a<\n\rBooleanSymbol\x12\x0e\n\x02id\x18\x01 \x02(\x05:\x02-1\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\x08\x1aI\n\nEnumSymbol\x12\x0e\n\x02id\x18\x01 \x02(\x05:\x02-1\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\r\n\x05value\x18\r \x01(\x05\x12\x0e\n\x06typeId\x18\x0e \x01(\x05\"\xff\x01\n\x13\x42\x65haviorStateSparse\x12\r\n\x05\x61gent\x18\x01 \x01(\t\x12;\n\x11\x61\x63tiveRootActions\x18\x02 \x03(\x0b\x32 .naothmessages.XABSLActionSparse\x12\x37\n\x0finputSymbolList\x18\n \x01(\x0b\x32\x1e.naothmessages.SymbolValueList\x12\x38\n\x10outputSymbolList\x18\x0b \x01(\x0b\x32\x1e.naothmessages.SymbolValueList\x12\x13\n\x0b\x66rameNumber\x18\x05 \x01(\r\x12\x14\n\x0c\x65rrorMessage\x18\x06 \x01(\t\"\xc8\x05\n\x15\x42\x65haviorStateComplete\x12\x43\n\x0c\x65numerations\x18\n \x03(\x0b\x32-.naothmessages.BehaviorStateComplete.EnumType\x12<\n\x07options\x18\x0b \x03(\x0b\x32+.naothmessages.BehaviorStateComplete.Option\x12:\n\x06\x61gents\x18\x0c \x03(\x0b\x32*.naothmessages.BehaviorStateComplete.Agent\x12\x37\n\x0finputSymbolList\x18\r \x01(\x0b\x32\x1e.naothmessages.SymbolValueList\x12\x38\n\x10outputSymbolList\x18\x0e \x01(\x0b\x32\x1e.naothmessages.SymbolValueList\x12\x13\n\x0b\x66rameNumber\x18\x0f \x01(\r\x1a\x89\x01\n\x08\x45numType\x12\x0c\n\x04name\x18\x01 \x02(\t\x12G\n\x08\x65lements\x18\x02 \x03(\x0b\x32\x35.naothmessages.BehaviorStateComplete.EnumType.Element\x1a&\n\x07\x45lement\x12\r\n\x05value\x18\x01 \x02(\x05\x12\x0c\n\x04name\x18\x02 \x02(\t\x1a\xb0\x01\n\x06Option\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x41\n\x06states\x18\x03 \x03(\x0b\x32\x31.naothmessages.BehaviorStateComplete.Option.State\x12.\n\nparameters\x18\x02 \x03(\x0b\x32\x1a.naothmessages.XABSLSymbol\x1a%\n\x05State\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x0e\n\x06target\x18\x02 \x01(\x08\x1a)\n\x05\x41gent\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x12\n\nrootOption\x18\x02 \x02(\t\"k\n\x06Module\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x1b\n\x13usedRepresentations\x18\x02 \x03(\t\x12\x1f\n\x17providedRepresentations\x18\x03 \x03(\t\x12\x15\n\x06\x61\x63tive\x18\x04 \x01(\x08:\x05\x66\x61lse\"4\n\nModuleList\x12&\n\x07modules\x18\x01 \x03(\x0b\x32\x15.naothmessages.Module\"\x83\x01\n\x0c\x44\x65\x62ugRequest\x12\x32\n\x08requests\x18\x01 \x03(\x0b\x32 .naothmessages.DebugRequest.Item\x1a?\n\x04Item\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x14\n\x05value\x18\x04 \x02(\x08:\x05\x66\x61lse\"\x8b\x01\n\x0f\x46\x65\x61tureKeyPoint\x12\'\n\x02pt\x18\x01 \x02(\x0b\x32\x1b.naothmessages.FloatVector2\x12\x0c\n\x04size\x18\x02 \x02(\x02\x12\r\n\x05\x61ngle\x18\x03 \x02(\x02\x12\x10\n\x08response\x18\x04 \x02(\x02\x12\x0e\n\x06octave\x18\x05 \x02(\x05\x12\x10\n\x08\x63lass_id\x18\x06 \x02(\x05\"\xd9\x01\n\x16\x46\x65\x61tureDetectionResult\x12\x16\n\x0egrayScaleImage\x18\x01 \x02(\x0c\x12\x31\n\tkeyPoints\x18\x02 \x03(\x0b\x32\x1e.naothmessages.FeatureKeyPoint\x12\x34\n\x0b\x64\x65scriptors\x18\x03 \x03(\x0b\x32\x1f.naothmessages.SingleDescriptor\x12\x19\n\x11namesForKeyPoints\x18\x04 \x03(\t\x12#\n\x17numberOfFeatureElements\x18\x05 \x01(\x05:\x02\x36\x34\"\x1d\n\x10SingleDescriptor\x12\t\n\x01\x64\x18\x01 \x03(\x02\"d\n\x0c\x46\x65\x61tureClass\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x30\n\x07samples\x18\x02 \x03(\x0b\x32\x1f.naothmessages.SingleDescriptor\x12\x14\n\x0cobjectHeight\x18\x03 \x01(\x01\"a\n\x0c\x46\x65\x61tureTable\x12,\n\x07\x63lasses\x18\x01 \x03(\x0b\x32\x1b.naothmessages.FeatureClass\x12#\n\x17numberOfFeatureElements\x18\x02 \x01(\x05:\x02\x36\x34\x42\x1b\n\x19\x64\x65.naoth.rc.core.messages')
  ,
  dependencies=[CommonTypes__pb2.DESCRIPTOR,])



_PLOTITEM_PLOTTYPE = _descriptor.EnumDescriptor(
  name='PlotType',
  full_name='naothmessages.PlotItem.PlotType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Default', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Plot2D', index=1, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Origin2D', index=2, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=390,
  serialized_end=439,
)
_sym_db.RegisterEnumDescriptor(_PLOTITEM_PLOTTYPE)

_XABSLSYMBOL_SYMBOLTYPE = _descriptor.EnumDescriptor(
  name='SymbolType',
  full_name='naothmessages.XABSLSymbol.SymbolType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Decimal', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Boolean', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Enum', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Unknown', index=3, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=804,
  serialized_end=865,
)
_sym_db.RegisterEnumDescriptor(_XABSLSYMBOL_SYMBOLTYPE)

_XABSLACTIONSPARSE_ACTIONTYPE = _descriptor.EnumDescriptor(
  name='ActionType',
  full_name='naothmessages.XABSLActionSparse.ActionType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Option', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='BasicBehavior', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SymbolAssignment', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='Unknown', index=3, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1313,
  serialized_end=1391,
)
_sym_db.RegisterEnumDescriptor(_XABSLACTIONSPARSE_ACTIONTYPE)


_CMDARG = _descriptor.Descriptor(
  name='CMDArg',
  full_name='naothmessages.CMDArg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='naothmessages.CMDArg.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bytes', full_name='naothmessages.CMDArg.bytes', index=1,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=52,
  serialized_end=89,
)


_CMD = _descriptor.Descriptor(
  name='CMD',
  full_name='naothmessages.CMD',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='naothmessages.CMD.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='args', full_name='naothmessages.CMD.args', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=91,
  serialized_end=147,
)


_STOPWATCHITEM = _descriptor.Descriptor(
  name='StopwatchItem',
  full_name='naothmessages.StopwatchItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='naothmessages.StopwatchItem.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='time', full_name='naothmessages.StopwatchItem.time', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=149,
  serialized_end=192,
)


_STOPWATCHES = _descriptor.Descriptor(
  name='Stopwatches',
  full_name='naothmessages.Stopwatches',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stopwatches', full_name='naothmessages.Stopwatches.stopwatches', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=194,
  serialized_end=258,
)


_PLOTITEM = _descriptor.Descriptor(
  name='PlotItem',
  full_name='naothmessages.PlotItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='naothmessages.PlotItem.type', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='naothmessages.PlotItem.name', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='naothmessages.PlotItem.value', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='x', full_name='naothmessages.PlotItem.x', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='y', full_name='naothmessages.PlotItem.y', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rotation', full_name='naothmessages.PlotItem.rotation', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _PLOTITEM_PLOTTYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=261,
  serialized_end=439,
)


_PLOTSTROKE2D = _descriptor.Descriptor(
  name='PlotStroke2D',
  full_name='naothmessages.PlotStroke2D',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='naothmessages.PlotStroke2D.name', index=0,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='points', full_name='naothmessages.PlotStroke2D.points', index=1,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=441,
  serialized_end=515,
)


_PLOTS = _descriptor.Descriptor(
  name='Plots',
  full_name='naothmessages.Plots',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='plots', full_name='naothmessages.Plots.plots', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='plotstrokes', full_name='naothmessages.Plots.plotstrokes', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=517,
  serialized_end=614,
)


_XABSLSYMBOL = _descriptor.Descriptor(
  name='XABSLSymbol',
  full_name='naothmessages.XABSLSymbol',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='naothmessages.XABSLSymbol.id', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=True, default_value=-1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='type', full_name='naothmessages.XABSLSymbol.type', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=True, default_value=3,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='naothmessages.XABSLSymbol.name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='decimalValue', full_name='naothmessages.XABSLSymbol.decimalValue', index=3,
      number=11, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='boolValue', full_name='naothmessages.XABSLSymbol.boolValue', index=4,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='enumValue', full_name='naothmessages.XABSLSymbol.enumValue', index=5,
      number=13, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='enumTypeId', full_name='naothmessages.XABSLSymbol.enumTypeId', index=6,
      number=14, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _XABSLSYMBOL_SYMBOLTYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=617,
  serialized_end=865,
)


_XABSLACTIVEOPTIONSPARSE = _descriptor.Descriptor(
  name='XABSLActiveOptionSparse',
  full_name='naothmessages.XABSLActiveOptionSparse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='naothmessages.XABSLActiveOptionSparse.id', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=True, default_value=-1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timeOfExecution', full_name='naothmessages.XABSLActiveOptionSparse.timeOfExecution', index=1,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='activeState', full_name='naothmessages.XABSLActiveOptionSparse.activeState', index=2,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='stateTime', full_name='naothmessages.XABSLActiveOptionSparse.stateTime', index=3,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='decimalParameters', full_name='naothmessages.XABSLActiveOptionSparse.decimalParameters', index=4,
      number=6, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='booleanParameters', full_name='naothmessages.XABSLActiveOptionSparse.booleanParameters', index=5,
      number=7, type=8, cpp_type=7, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='enumeratedParameters', full_name='naothmessages.XABSLActiveOptionSparse.enumeratedParameters', index=6,
      number=8, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='activeSubActions', full_name='naothmessages.XABSLActiveOptionSparse.activeSubActions', index=7,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=868,
  serialized_end=1121,
)


_XABSLACTIONSPARSE = _descriptor.Descriptor(
  name='XABSLActionSparse',
  full_name='naothmessages.XABSLActionSparse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='naothmessages.XABSLActionSparse.type', index=0,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=True, default_value=3,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='symbol', full_name='naothmessages.XABSLActionSparse.symbol', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='option', full_name='naothmessages.XABSLActionSparse.option', index=2,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _XABSLACTIONSPARSE_ACTIONTYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1124,
  serialized_end=1391,
)


_SYMBOLVALUELIST_DOUBLESYMBOL = _descriptor.Descriptor(
  name='DoubleSymbol',
  full_name='naothmessages.SymbolValueList.DoubleSymbol',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='naothmessages.SymbolValueList.DoubleSymbol.id', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=True, default_value=-1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='naothmessages.SymbolValueList.DoubleSymbol.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='naothmessages.SymbolValueList.DoubleSymbol.value', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1601,
  serialized_end=1660,
)

_SYMBOLVALUELIST_BOOLEANSYMBOL = _descriptor.Descriptor(
  name='BooleanSymbol',
  full_name='naothmessages.SymbolValueList.BooleanSymbol',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='naothmessages.SymbolValueList.BooleanSymbol.id', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=True, default_value=-1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='naothmessages.SymbolValueList.BooleanSymbol.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='naothmessages.SymbolValueList.BooleanSymbol.value', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1662,
  serialized_end=1722,
)

_SYMBOLVALUELIST_ENUMSYMBOL = _descriptor.Descriptor(
  name='EnumSymbol',
  full_name='naothmessages.SymbolValueList.EnumSymbol',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='naothmessages.SymbolValueList.EnumSymbol.id', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=True, default_value=-1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='naothmessages.SymbolValueList.EnumSymbol.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='naothmessages.SymbolValueList.EnumSymbol.value', index=2,
      number=13, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='typeId', full_name='naothmessages.SymbolValueList.EnumSymbol.typeId', index=3,
      number=14, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1724,
  serialized_end=1797,
)

_SYMBOLVALUELIST = _descriptor.Descriptor(
  name='SymbolValueList',
  full_name='naothmessages.SymbolValueList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='decimal', full_name='naothmessages.SymbolValueList.decimal', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='boolean', full_name='naothmessages.SymbolValueList.boolean', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='enumerated', full_name='naothmessages.SymbolValueList.enumerated', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_SYMBOLVALUELIST_DOUBLESYMBOL, _SYMBOLVALUELIST_BOOLEANSYMBOL, _SYMBOLVALUELIST_ENUMSYMBOL, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1394,
  serialized_end=1797,
)


_BEHAVIORSTATESPARSE = _descriptor.Descriptor(
  name='BehaviorStateSparse',
  full_name='naothmessages.BehaviorStateSparse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='agent', full_name='naothmessages.BehaviorStateSparse.agent', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='activeRootActions', full_name='naothmessages.BehaviorStateSparse.activeRootActions', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='inputSymbolList', full_name='naothmessages.BehaviorStateSparse.inputSymbolList', index=2,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='outputSymbolList', full_name='naothmessages.BehaviorStateSparse.outputSymbolList', index=3,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='frameNumber', full_name='naothmessages.BehaviorStateSparse.frameNumber', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='errorMessage', full_name='naothmessages.BehaviorStateSparse.errorMessage', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1800,
  serialized_end=2055,
)


_BEHAVIORSTATECOMPLETE_ENUMTYPE_ELEMENT = _descriptor.Descriptor(
  name='Element',
  full_name='naothmessages.BehaviorStateComplete.EnumType.Element',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='naothmessages.BehaviorStateComplete.EnumType.Element.value', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='naothmessages.BehaviorStateComplete.EnumType.Element.name', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2510,
  serialized_end=2548,
)

_BEHAVIORSTATECOMPLETE_ENUMTYPE = _descriptor.Descriptor(
  name='EnumType',
  full_name='naothmessages.BehaviorStateComplete.EnumType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='naothmessages.BehaviorStateComplete.EnumType.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='elements', full_name='naothmessages.BehaviorStateComplete.EnumType.elements', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_BEHAVIORSTATECOMPLETE_ENUMTYPE_ELEMENT, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2411,
  serialized_end=2548,
)

_BEHAVIORSTATECOMPLETE_OPTION_STATE = _descriptor.Descriptor(
  name='State',
  full_name='naothmessages.BehaviorStateComplete.Option.State',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='naothmessages.BehaviorStateComplete.Option.State.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='target', full_name='naothmessages.BehaviorStateComplete.Option.State.target', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2690,
  serialized_end=2727,
)

_BEHAVIORSTATECOMPLETE_OPTION = _descriptor.Descriptor(
  name='Option',
  full_name='naothmessages.BehaviorStateComplete.Option',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='naothmessages.BehaviorStateComplete.Option.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='states', full_name='naothmessages.BehaviorStateComplete.Option.states', index=1,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='parameters', full_name='naothmessages.BehaviorStateComplete.Option.parameters', index=2,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_BEHAVIORSTATECOMPLETE_OPTION_STATE, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2551,
  serialized_end=2727,
)

_BEHAVIORSTATECOMPLETE_AGENT = _descriptor.Descriptor(
  name='Agent',
  full_name='naothmessages.BehaviorStateComplete.Agent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='naothmessages.BehaviorStateComplete.Agent.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rootOption', full_name='naothmessages.BehaviorStateComplete.Agent.rootOption', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2729,
  serialized_end=2770,
)

_BEHAVIORSTATECOMPLETE = _descriptor.Descriptor(
  name='BehaviorStateComplete',
  full_name='naothmessages.BehaviorStateComplete',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='enumerations', full_name='naothmessages.BehaviorStateComplete.enumerations', index=0,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='options', full_name='naothmessages.BehaviorStateComplete.options', index=1,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='agents', full_name='naothmessages.BehaviorStateComplete.agents', index=2,
      number=12, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='inputSymbolList', full_name='naothmessages.BehaviorStateComplete.inputSymbolList', index=3,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='outputSymbolList', full_name='naothmessages.BehaviorStateComplete.outputSymbolList', index=4,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='frameNumber', full_name='naothmessages.BehaviorStateComplete.frameNumber', index=5,
      number=15, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_BEHAVIORSTATECOMPLETE_ENUMTYPE, _BEHAVIORSTATECOMPLETE_OPTION, _BEHAVIORSTATECOMPLETE_AGENT, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2058,
  serialized_end=2770,
)


_MODULE = _descriptor.Descriptor(
  name='Module',
  full_name='naothmessages.Module',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='naothmessages.Module.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='usedRepresentations', full_name='naothmessages.Module.usedRepresentations', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='providedRepresentations', full_name='naothmessages.Module.providedRepresentations', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='active', full_name='naothmessages.Module.active', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2772,
  serialized_end=2879,
)


_MODULELIST = _descriptor.Descriptor(
  name='ModuleList',
  full_name='naothmessages.ModuleList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='modules', full_name='naothmessages.ModuleList.modules', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2881,
  serialized_end=2933,
)


_DEBUGREQUEST_ITEM = _descriptor.Descriptor(
  name='Item',
  full_name='naothmessages.DebugRequest.Item',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='naothmessages.DebugRequest.Item.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='description', full_name='naothmessages.DebugRequest.Item.description', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='naothmessages.DebugRequest.Item.value', index=2,
      number=4, type=8, cpp_type=7, label=2,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3004,
  serialized_end=3067,
)

_DEBUGREQUEST = _descriptor.Descriptor(
  name='DebugRequest',
  full_name='naothmessages.DebugRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='requests', full_name='naothmessages.DebugRequest.requests', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_DEBUGREQUEST_ITEM, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2936,
  serialized_end=3067,
)


_FEATUREKEYPOINT = _descriptor.Descriptor(
  name='FeatureKeyPoint',
  full_name='naothmessages.FeatureKeyPoint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pt', full_name='naothmessages.FeatureKeyPoint.pt', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='size', full_name='naothmessages.FeatureKeyPoint.size', index=1,
      number=2, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='angle', full_name='naothmessages.FeatureKeyPoint.angle', index=2,
      number=3, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='response', full_name='naothmessages.FeatureKeyPoint.response', index=3,
      number=4, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='octave', full_name='naothmessages.FeatureKeyPoint.octave', index=4,
      number=5, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='class_id', full_name='naothmessages.FeatureKeyPoint.class_id', index=5,
      number=6, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3070,
  serialized_end=3209,
)


_FEATUREDETECTIONRESULT = _descriptor.Descriptor(
  name='FeatureDetectionResult',
  full_name='naothmessages.FeatureDetectionResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='grayScaleImage', full_name='naothmessages.FeatureDetectionResult.grayScaleImage', index=0,
      number=1, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='keyPoints', full_name='naothmessages.FeatureDetectionResult.keyPoints', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='descriptors', full_name='naothmessages.FeatureDetectionResult.descriptors', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='namesForKeyPoints', full_name='naothmessages.FeatureDetectionResult.namesForKeyPoints', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='numberOfFeatureElements', full_name='naothmessages.FeatureDetectionResult.numberOfFeatureElements', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=64,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3212,
  serialized_end=3429,
)


_SINGLEDESCRIPTOR = _descriptor.Descriptor(
  name='SingleDescriptor',
  full_name='naothmessages.SingleDescriptor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='d', full_name='naothmessages.SingleDescriptor.d', index=0,
      number=1, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3431,
  serialized_end=3460,
)


_FEATURECLASS = _descriptor.Descriptor(
  name='FeatureClass',
  full_name='naothmessages.FeatureClass',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='naothmessages.FeatureClass.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='samples', full_name='naothmessages.FeatureClass.samples', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='objectHeight', full_name='naothmessages.FeatureClass.objectHeight', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3462,
  serialized_end=3562,
)


_FEATURETABLE = _descriptor.Descriptor(
  name='FeatureTable',
  full_name='naothmessages.FeatureTable',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='classes', full_name='naothmessages.FeatureTable.classes', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='numberOfFeatureElements', full_name='naothmessages.FeatureTable.numberOfFeatureElements', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=64,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3564,
  serialized_end=3661,
)

_CMD.fields_by_name['args'].message_type = _CMDARG
_STOPWATCHES.fields_by_name['stopwatches'].message_type = _STOPWATCHITEM
_PLOTITEM.fields_by_name['type'].enum_type = _PLOTITEM_PLOTTYPE
_PLOTITEM_PLOTTYPE.containing_type = _PLOTITEM
_PLOTSTROKE2D.fields_by_name['points'].message_type = CommonTypes__pb2._DOUBLEVECTOR2
_PLOTS.fields_by_name['plots'].message_type = _PLOTITEM
_PLOTS.fields_by_name['plotstrokes'].message_type = _PLOTSTROKE2D
_XABSLSYMBOL.fields_by_name['type'].enum_type = _XABSLSYMBOL_SYMBOLTYPE
_XABSLSYMBOL_SYMBOLTYPE.containing_type = _XABSLSYMBOL
_XABSLACTIVEOPTIONSPARSE.fields_by_name['activeSubActions'].message_type = _XABSLACTIONSPARSE
_XABSLACTIONSPARSE.fields_by_name['type'].enum_type = _XABSLACTIONSPARSE_ACTIONTYPE
_XABSLACTIONSPARSE.fields_by_name['symbol'].message_type = _XABSLSYMBOL
_XABSLACTIONSPARSE.fields_by_name['option'].message_type = _XABSLACTIVEOPTIONSPARSE
_XABSLACTIONSPARSE_ACTIONTYPE.containing_type = _XABSLACTIONSPARSE
_SYMBOLVALUELIST_DOUBLESYMBOL.containing_type = _SYMBOLVALUELIST
_SYMBOLVALUELIST_BOOLEANSYMBOL.containing_type = _SYMBOLVALUELIST
_SYMBOLVALUELIST_ENUMSYMBOL.containing_type = _SYMBOLVALUELIST
_SYMBOLVALUELIST.fields_by_name['decimal'].message_type = _SYMBOLVALUELIST_DOUBLESYMBOL
_SYMBOLVALUELIST.fields_by_name['boolean'].message_type = _SYMBOLVALUELIST_BOOLEANSYMBOL
_SYMBOLVALUELIST.fields_by_name['enumerated'].message_type = _SYMBOLVALUELIST_ENUMSYMBOL
_BEHAVIORSTATESPARSE.fields_by_name['activeRootActions'].message_type = _XABSLACTIONSPARSE
_BEHAVIORSTATESPARSE.fields_by_name['inputSymbolList'].message_type = _SYMBOLVALUELIST
_BEHAVIORSTATESPARSE.fields_by_name['outputSymbolList'].message_type = _SYMBOLVALUELIST
_BEHAVIORSTATECOMPLETE_ENUMTYPE_ELEMENT.containing_type = _BEHAVIORSTATECOMPLETE_ENUMTYPE
_BEHAVIORSTATECOMPLETE_ENUMTYPE.fields_by_name['elements'].message_type = _BEHAVIORSTATECOMPLETE_ENUMTYPE_ELEMENT
_BEHAVIORSTATECOMPLETE_ENUMTYPE.containing_type = _BEHAVIORSTATECOMPLETE
_BEHAVIORSTATECOMPLETE_OPTION_STATE.containing_type = _BEHAVIORSTATECOMPLETE_OPTION
_BEHAVIORSTATECOMPLETE_OPTION.fields_by_name['states'].message_type = _BEHAVIORSTATECOMPLETE_OPTION_STATE
_BEHAVIORSTATECOMPLETE_OPTION.fields_by_name['parameters'].message_type = _XABSLSYMBOL
_BEHAVIORSTATECOMPLETE_OPTION.containing_type = _BEHAVIORSTATECOMPLETE
_BEHAVIORSTATECOMPLETE_AGENT.containing_type = _BEHAVIORSTATECOMPLETE
_BEHAVIORSTATECOMPLETE.fields_by_name['enumerations'].message_type = _BEHAVIORSTATECOMPLETE_ENUMTYPE
_BEHAVIORSTATECOMPLETE.fields_by_name['options'].message_type = _BEHAVIORSTATECOMPLETE_OPTION
_BEHAVIORSTATECOMPLETE.fields_by_name['agents'].message_type = _BEHAVIORSTATECOMPLETE_AGENT
_BEHAVIORSTATECOMPLETE.fields_by_name['inputSymbolList'].message_type = _SYMBOLVALUELIST
_BEHAVIORSTATECOMPLETE.fields_by_name['outputSymbolList'].message_type = _SYMBOLVALUELIST
_MODULELIST.fields_by_name['modules'].message_type = _MODULE
_DEBUGREQUEST_ITEM.containing_type = _DEBUGREQUEST
_DEBUGREQUEST.fields_by_name['requests'].message_type = _DEBUGREQUEST_ITEM
_FEATUREKEYPOINT.fields_by_name['pt'].message_type = CommonTypes__pb2._FLOATVECTOR2
_FEATUREDETECTIONRESULT.fields_by_name['keyPoints'].message_type = _FEATUREKEYPOINT
_FEATUREDETECTIONRESULT.fields_by_name['descriptors'].message_type = _SINGLEDESCRIPTOR
_FEATURECLASS.fields_by_name['samples'].message_type = _SINGLEDESCRIPTOR
_FEATURETABLE.fields_by_name['classes'].message_type = _FEATURECLASS
DESCRIPTOR.message_types_by_name['CMDArg'] = _CMDARG
DESCRIPTOR.message_types_by_name['CMD'] = _CMD
DESCRIPTOR.message_types_by_name['StopwatchItem'] = _STOPWATCHITEM
DESCRIPTOR.message_types_by_name['Stopwatches'] = _STOPWATCHES
DESCRIPTOR.message_types_by_name['PlotItem'] = _PLOTITEM
DESCRIPTOR.message_types_by_name['PlotStroke2D'] = _PLOTSTROKE2D
DESCRIPTOR.message_types_by_name['Plots'] = _PLOTS
DESCRIPTOR.message_types_by_name['XABSLSymbol'] = _XABSLSYMBOL
DESCRIPTOR.message_types_by_name['XABSLActiveOptionSparse'] = _XABSLACTIVEOPTIONSPARSE
DESCRIPTOR.message_types_by_name['XABSLActionSparse'] = _XABSLACTIONSPARSE
DESCRIPTOR.message_types_by_name['SymbolValueList'] = _SYMBOLVALUELIST
DESCRIPTOR.message_types_by_name['BehaviorStateSparse'] = _BEHAVIORSTATESPARSE
DESCRIPTOR.message_types_by_name['BehaviorStateComplete'] = _BEHAVIORSTATECOMPLETE
DESCRIPTOR.message_types_by_name['Module'] = _MODULE
DESCRIPTOR.message_types_by_name['ModuleList'] = _MODULELIST
DESCRIPTOR.message_types_by_name['DebugRequest'] = _DEBUGREQUEST
DESCRIPTOR.message_types_by_name['FeatureKeyPoint'] = _FEATUREKEYPOINT
DESCRIPTOR.message_types_by_name['FeatureDetectionResult'] = _FEATUREDETECTIONRESULT
DESCRIPTOR.message_types_by_name['SingleDescriptor'] = _SINGLEDESCRIPTOR
DESCRIPTOR.message_types_by_name['FeatureClass'] = _FEATURECLASS
DESCRIPTOR.message_types_by_name['FeatureTable'] = _FEATURETABLE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CMDArg = _reflection.GeneratedProtocolMessageType('CMDArg', (_message.Message,), dict(
  DESCRIPTOR = _CMDARG,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.CMDArg)
  ))
_sym_db.RegisterMessage(CMDArg)

CMD = _reflection.GeneratedProtocolMessageType('CMD', (_message.Message,), dict(
  DESCRIPTOR = _CMD,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.CMD)
  ))
_sym_db.RegisterMessage(CMD)

StopwatchItem = _reflection.GeneratedProtocolMessageType('StopwatchItem', (_message.Message,), dict(
  DESCRIPTOR = _STOPWATCHITEM,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.StopwatchItem)
  ))
_sym_db.RegisterMessage(StopwatchItem)

Stopwatches = _reflection.GeneratedProtocolMessageType('Stopwatches', (_message.Message,), dict(
  DESCRIPTOR = _STOPWATCHES,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.Stopwatches)
  ))
_sym_db.RegisterMessage(Stopwatches)

PlotItem = _reflection.GeneratedProtocolMessageType('PlotItem', (_message.Message,), dict(
  DESCRIPTOR = _PLOTITEM,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.PlotItem)
  ))
_sym_db.RegisterMessage(PlotItem)

PlotStroke2D = _reflection.GeneratedProtocolMessageType('PlotStroke2D', (_message.Message,), dict(
  DESCRIPTOR = _PLOTSTROKE2D,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.PlotStroke2D)
  ))
_sym_db.RegisterMessage(PlotStroke2D)

Plots = _reflection.GeneratedProtocolMessageType('Plots', (_message.Message,), dict(
  DESCRIPTOR = _PLOTS,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.Plots)
  ))
_sym_db.RegisterMessage(Plots)

XABSLSymbol = _reflection.GeneratedProtocolMessageType('XABSLSymbol', (_message.Message,), dict(
  DESCRIPTOR = _XABSLSYMBOL,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.XABSLSymbol)
  ))
_sym_db.RegisterMessage(XABSLSymbol)

XABSLActiveOptionSparse = _reflection.GeneratedProtocolMessageType('XABSLActiveOptionSparse', (_message.Message,), dict(
  DESCRIPTOR = _XABSLACTIVEOPTIONSPARSE,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.XABSLActiveOptionSparse)
  ))
_sym_db.RegisterMessage(XABSLActiveOptionSparse)

XABSLActionSparse = _reflection.GeneratedProtocolMessageType('XABSLActionSparse', (_message.Message,), dict(
  DESCRIPTOR = _XABSLACTIONSPARSE,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.XABSLActionSparse)
  ))
_sym_db.RegisterMessage(XABSLActionSparse)

SymbolValueList = _reflection.GeneratedProtocolMessageType('SymbolValueList', (_message.Message,), dict(

  DoubleSymbol = _reflection.GeneratedProtocolMessageType('DoubleSymbol', (_message.Message,), dict(
    DESCRIPTOR = _SYMBOLVALUELIST_DOUBLESYMBOL,
    __module__ = 'Messages_pb2'
    # @@protoc_insertion_point(class_scope:naothmessages.SymbolValueList.DoubleSymbol)
    ))
  ,

  BooleanSymbol = _reflection.GeneratedProtocolMessageType('BooleanSymbol', (_message.Message,), dict(
    DESCRIPTOR = _SYMBOLVALUELIST_BOOLEANSYMBOL,
    __module__ = 'Messages_pb2'
    # @@protoc_insertion_point(class_scope:naothmessages.SymbolValueList.BooleanSymbol)
    ))
  ,

  EnumSymbol = _reflection.GeneratedProtocolMessageType('EnumSymbol', (_message.Message,), dict(
    DESCRIPTOR = _SYMBOLVALUELIST_ENUMSYMBOL,
    __module__ = 'Messages_pb2'
    # @@protoc_insertion_point(class_scope:naothmessages.SymbolValueList.EnumSymbol)
    ))
  ,
  DESCRIPTOR = _SYMBOLVALUELIST,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.SymbolValueList)
  ))
_sym_db.RegisterMessage(SymbolValueList)
_sym_db.RegisterMessage(SymbolValueList.DoubleSymbol)
_sym_db.RegisterMessage(SymbolValueList.BooleanSymbol)
_sym_db.RegisterMessage(SymbolValueList.EnumSymbol)

BehaviorStateSparse = _reflection.GeneratedProtocolMessageType('BehaviorStateSparse', (_message.Message,), dict(
  DESCRIPTOR = _BEHAVIORSTATESPARSE,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.BehaviorStateSparse)
  ))
_sym_db.RegisterMessage(BehaviorStateSparse)

BehaviorStateComplete = _reflection.GeneratedProtocolMessageType('BehaviorStateComplete', (_message.Message,), dict(

  EnumType = _reflection.GeneratedProtocolMessageType('EnumType', (_message.Message,), dict(

    Element = _reflection.GeneratedProtocolMessageType('Element', (_message.Message,), dict(
      DESCRIPTOR = _BEHAVIORSTATECOMPLETE_ENUMTYPE_ELEMENT,
      __module__ = 'Messages_pb2'
      # @@protoc_insertion_point(class_scope:naothmessages.BehaviorStateComplete.EnumType.Element)
      ))
    ,
    DESCRIPTOR = _BEHAVIORSTATECOMPLETE_ENUMTYPE,
    __module__ = 'Messages_pb2'
    # @@protoc_insertion_point(class_scope:naothmessages.BehaviorStateComplete.EnumType)
    ))
  ,

  Option = _reflection.GeneratedProtocolMessageType('Option', (_message.Message,), dict(

    State = _reflection.GeneratedProtocolMessageType('State', (_message.Message,), dict(
      DESCRIPTOR = _BEHAVIORSTATECOMPLETE_OPTION_STATE,
      __module__ = 'Messages_pb2'
      # @@protoc_insertion_point(class_scope:naothmessages.BehaviorStateComplete.Option.State)
      ))
    ,
    DESCRIPTOR = _BEHAVIORSTATECOMPLETE_OPTION,
    __module__ = 'Messages_pb2'
    # @@protoc_insertion_point(class_scope:naothmessages.BehaviorStateComplete.Option)
    ))
  ,

  Agent = _reflection.GeneratedProtocolMessageType('Agent', (_message.Message,), dict(
    DESCRIPTOR = _BEHAVIORSTATECOMPLETE_AGENT,
    __module__ = 'Messages_pb2'
    # @@protoc_insertion_point(class_scope:naothmessages.BehaviorStateComplete.Agent)
    ))
  ,
  DESCRIPTOR = _BEHAVIORSTATECOMPLETE,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.BehaviorStateComplete)
  ))
_sym_db.RegisterMessage(BehaviorStateComplete)
_sym_db.RegisterMessage(BehaviorStateComplete.EnumType)
_sym_db.RegisterMessage(BehaviorStateComplete.EnumType.Element)
_sym_db.RegisterMessage(BehaviorStateComplete.Option)
_sym_db.RegisterMessage(BehaviorStateComplete.Option.State)
_sym_db.RegisterMessage(BehaviorStateComplete.Agent)

Module = _reflection.GeneratedProtocolMessageType('Module', (_message.Message,), dict(
  DESCRIPTOR = _MODULE,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.Module)
  ))
_sym_db.RegisterMessage(Module)

ModuleList = _reflection.GeneratedProtocolMessageType('ModuleList', (_message.Message,), dict(
  DESCRIPTOR = _MODULELIST,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.ModuleList)
  ))
_sym_db.RegisterMessage(ModuleList)

DebugRequest = _reflection.GeneratedProtocolMessageType('DebugRequest', (_message.Message,), dict(

  Item = _reflection.GeneratedProtocolMessageType('Item', (_message.Message,), dict(
    DESCRIPTOR = _DEBUGREQUEST_ITEM,
    __module__ = 'Messages_pb2'
    # @@protoc_insertion_point(class_scope:naothmessages.DebugRequest.Item)
    ))
  ,
  DESCRIPTOR = _DEBUGREQUEST,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.DebugRequest)
  ))
_sym_db.RegisterMessage(DebugRequest)
_sym_db.RegisterMessage(DebugRequest.Item)

FeatureKeyPoint = _reflection.GeneratedProtocolMessageType('FeatureKeyPoint', (_message.Message,), dict(
  DESCRIPTOR = _FEATUREKEYPOINT,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.FeatureKeyPoint)
  ))
_sym_db.RegisterMessage(FeatureKeyPoint)

FeatureDetectionResult = _reflection.GeneratedProtocolMessageType('FeatureDetectionResult', (_message.Message,), dict(
  DESCRIPTOR = _FEATUREDETECTIONRESULT,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.FeatureDetectionResult)
  ))
_sym_db.RegisterMessage(FeatureDetectionResult)

SingleDescriptor = _reflection.GeneratedProtocolMessageType('SingleDescriptor', (_message.Message,), dict(
  DESCRIPTOR = _SINGLEDESCRIPTOR,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.SingleDescriptor)
  ))
_sym_db.RegisterMessage(SingleDescriptor)

FeatureClass = _reflection.GeneratedProtocolMessageType('FeatureClass', (_message.Message,), dict(
  DESCRIPTOR = _FEATURECLASS,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.FeatureClass)
  ))
_sym_db.RegisterMessage(FeatureClass)

FeatureTable = _reflection.GeneratedProtocolMessageType('FeatureTable', (_message.Message,), dict(
  DESCRIPTOR = _FEATURETABLE,
  __module__ = 'Messages_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.FeatureTable)
  ))
_sym_db.RegisterMessage(FeatureTable)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\031de.naoth.rc.core.messages'))
# @@protoc_insertion_point(module_scope)



import math as _math
import numbers as _numbers


def clamp(x, minimum, maximum):
    # function f where f(x)=x if min<= x <= max, f(x) = minimum if x < minimum, f(x) = maximum if x > maximum
    return max(minimum, min(x, maximum))


class Vector2:
    def __init__(self, x=0.0, y=0.0, pb_vector=None):
        if pb_vector is not None:
            self.x = pb_vector.x
            self.y = pb_vector.y
        else:
            self.x = x
            self.y = y

    def __add__(self, other):
        return Vector2(self.x + other.x, self.y + other.y)

    def __sub__(self, other):
        return Vector2(self.x - other.x, self.y - other.y)

    def __neg__(self):
        return Vector2(-self.x, -self.y)

    def rotate(self, a):
        return Vector2(
            _math.cos(a) * self.x - _math.sin(a) * self.y,
            _math.sin(a) * self.x + _math.cos(a) * self.y)

    def abs(self):
        return _math.sqrt(self.x * self.x + self.y * self.y)

    def __mul__(self, other):
        if isinstance(other, Vector2):
            # interpret multiplication as other^t * self (scalar product of the vectors, where ^t means transpose)
            return self.x * other.x + self.y * other.y
        elif isinstance(other, _numbers.Number):
            return Vector2(self.x * other, self.y * other)
        else:
            return NotImplemented

    def __rmul__(self, other):
        if isinstance(other, Vector2):
            # interpret multiplication as other^t * self (scalar product of the vectors, where ^t means transpose)
            return self.x * other.x + self.y * other.y
        elif isinstance(other, _numbers.Number):
            return Vector2(self.x * other, self.y * other)
        else:
            return NotImplemented

    # TODO python3: use __truediv__
    # https://docs.python.org/3/library/operator.html
    def __div__(self, fraction):
        return Vector2(self.x / fraction, self.y / fraction)

    def __truediv__(self, fraction):
        return Vector2(self.x / fraction, self.y / fraction)

    def __str__(self):
        return "({0},{1})".format(self.x, self.y)

    def __repr__(self):
        return str(self)

    def __eq__(self, other):
        if self.x == other.x and self.y == other.y:
            return True
        else:
            return False

    def __getitem__(self, k):
        if k == 0:
            return self.x
        elif k == 1:
            return self.y
        else:
            raise IndexError()

    def normalize(self):
        if Vector2.abs(self) != 0:
            return self / Vector2.abs(self)
        else:
            return self

    def normalize_length(self, length):
        if self.x == 0 and self.y == 0:
            return Vector2(self.x, self.y)
        else:
            return (Vector2(self.x, self.y) * length) / self.abs()

    # TODO is this function really necessary?
    def rotate_right(self):
        x = self.x
        y = self.y
        return Vector2(y, -x)

    def angle(self):
        return _math.atan2(self.y, self.x)

    def copy(self):
        return Vector2(self.x, self.y)


class Pose2D:
    def __init__(self, translation=Vector2(), rotation=0, pb_pose=None):
        if pb_pose is not None:
            self.translation = Vector2(pb_vector=pb_pose.translation)
            self.rotation = pb_pose.rotation
        else:
            self.translation = translation.copy()
            self.rotation = rotation

    def __mul__(self, other):
        if isinstance(other, Vector2):
            return other.rotate(self.rotation) + self.translation
        elif isinstance(other, Pose2D):
            p = Pose2D()
            p.translation = self * other.translation
            p.rotation = (self.rotation + other.rotation + _math.pi) % (2 * _math.pi) - _math.pi
            return p
        else:
            return NotImplemented

    def __invert__(self):
        return self.invert()

    def invert(self):
        p = Pose2D()
        p.rotation = -self.rotation
        p.translation = (Vector2() - self.translation).rotate(p.rotation)
        return p

    # operator method for '/' in python2
    def __div__(self, point):
        if isinstance(point, Vector2):
            return (point - self.translation).rotate(-self.rotation)
        else:
            return NotImplemented

    # operator method for '/' in python3
    def __truediv__(self, point):
        if isinstance(point, Vector2):
            return (point - self.translation).rotate(-self.rotation)
        else:
            return NotImplemented

    def __str__(self):
        return "(translation = {0}, rotation = {1})".format(self.translation, self.rotation)

    def __repr__(self):
        return str(self)

    def rotate(self, a):
        self.rotation += a

    def translate(self, x, y):
        self.translation = self * Vector2(x, y)

    def copy(self):
        return Pose2D(self.translation.copy(), self.rotation)


class LineSegment(object):
    def __init__(self, begin=Vector2(), end=Vector2()):
        self.base = begin
        self.direction = end - self.base
        self.length = Vector2.abs(self.direction)
        if self.direction.abs() != 0:
            self.direction /= self.direction.abs()

    def __str__(self):
        return str("Begin: " + str(self.base)) + " End: " + str(self.base + self.direction * self.length)

    def begin(self):
        return self.base

    def end(self):
        return self.base + self.direction * self.length

    def point(self, t):
        # waere es nicht besser hier auch noch zu normieren, damit man t in [0,1] waehlen kann, mit t = 0 -> base point
        # t = 1 -> end point
        t = clamp(t, 0.0, self.length)
        return self.base + self.direction * t

    def project(self, p):
        return self.direction * p - self.direction * self.base

    def projection(self, p):
        t = self.direction * p - self.direction * self.base
        return self.point(t)

    def intersection(self, other):
        # Difference to line_intersection is the additional check if t is within the length of the linesegment.
        # HACK returns linesegment distance if intersection is not on linesegment

        normal = Vector2(-other.direction.y, other.direction.x)
        t = normal * self.direction
        if t == 0:
            return float('Inf')

        t = normal * (other.base - self.base) / t
        t = clamp(t, 0.0, self.length)

        return t

    def line_intersection(self, other):
        # maybe rename methods, this one and the one above; the return ist on how to scale the direction
        # vector to get the intersect point from the start (base) point. this is not derivable from the name
        normal = Vector2(-other.direction.y, other.direction.x)
        t = normal * self.direction
        if t == 0:
            return float('Inf')
        else:
            return normal * (other.base - self.base) / t

    def intersect(self, other):
        t = self.line_intersection(other)
        return 0.0 <= t <= self.length


class Rect2d:
    def __init__(self, a, b):
        self.minimum = Vector2(min(a.x, b.x), min(a.y, b.y))
        self.maximum = Vector2(max(a.x, b.x), max(a.y, b.y))

    def inside(self, p):
        return self.maximum.x >= p.x >= self.minimum.x and self.maximum.y >= p.y >= self.minimum.y

    def min(self):
        return self.minimum

    def max(self):
        return self.maximum




def load_data():
    from . import __get_file
    base = 'https://www2.informatik.hu-berlin.de/~naoth/ressources/log/demo_jointdata/'
    file = ('collision_goal_left1.log', '23dba98265076edb8951db79e914e501')
    return __get_file(file[0], file_hash=file[1], origin=base, cache_subdir='demo_jointdata')




def load_data():
    from . import __get_file
    base = 'https://www2.informatik.hu-berlin.de/~naoth/ressources/log/demo_fsr/'
    files = [
        ("fallen_motion.log", 'cf94b2af23c5d7cedea45f5cb0916048'),
        ("sit_motion.log", '5fb0e5dc5d7d634b458a38e54ab30bff'),
        ("stand_motion.log", 'c87f91a9cc6fde60365865e38e70867a'),
        ("walk_motion.log", '71f68a978985f3b2df114c6624e3253a')
    ]

    return [ __get_file(f, origin=base, file_hash=h, cache_subdir='demo_fsr') for f, h in files ]



r"""Command-line tool to start a debug proxy.

usage: python -m naoth.utils.DebugProxy host [port] [--target TARGET] [--print]
"""
import argparse

from . import DebugProxy


if __name__ == '__main__':
    prog = 'python -m naoth.utils.DebugProxy'
    description = 'A simple command line tool to start a naoth debug proxy.'
    parser = argparse.ArgumentParser(prog=prog, description=description)
    parser.add_argument('host',
                        help='The host of the naoth agent (eg. "localhost")')
    parser.add_argument('port', nargs='?', type=int, default=5401,
                        help='The debug port of the naoth agent (default: 5401)')
    parser.add_argument('--target', type=int, default=7777,
                        help='The proxy port where other applications can connect to (default: 7777)')
    parser.add_argument('--print', action='store_true', default=False,
                        help='Print all commands and their responses going through the proxy.')
    args = parser.parse_args()

    p = DebugProxy(args.host, args.port, args.target, args.print)
    print('DebugProxy active at {}:{} <-> localhost:{}'.format(args.host, args.port, args.target))
    print('Ctrl+C to stop the proxy ...')
    try:
        p.join()
    except KeyboardInterrupt:
        pass
    finally:
        print('\nBye')
        p.stop()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: CommonTypes.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='CommonTypes.proto',
  package='naothmessages',
  syntax='proto2',
  serialized_pb=_b('\n\x11\x43ommonTypes.proto\x12\rnaothmessages\"$\n\x0c\x46loatVector2\x12\t\n\x01x\x18\x01 \x02(\x02\x12\t\n\x01y\x18\x02 \x02(\x02\"%\n\rDoubleVector2\x12\t\n\x01x\x18\x01 \x02(\x01\x12\t\n\x01y\x18\x02 \x02(\x01\"\"\n\nIntVector2\x12\t\n\x01x\x18\x01 \x02(\x05\x12\t\n\x01y\x18\x02 \x02(\x05\"M\n\x06Pose2D\x12\x31\n\x0btranslation\x18\x01 \x02(\x0b\x32\x1c.naothmessages.DoubleVector2\x12\x10\n\x08rotation\x18\x02 \x02(\x01\"0\n\rDoubleVector3\x12\t\n\x01x\x18\x01 \x02(\x01\x12\t\n\x01y\x18\x02 \x02(\x01\x12\t\n\x01z\x18\x03 \x02(\x01\"\x19\n\x0c\x44oubleVector\x12\t\n\x01v\x18\x01 \x03(\x01\"k\n\x06Pose3D\x12\x31\n\x0btranslation\x18\x01 \x02(\x0b\x32\x1c.naothmessages.DoubleVector3\x12.\n\x08rotation\x18\x02 \x03(\x0b\x32\x1c.naothmessages.DoubleVector3\"z\n\x0bLineSegment\x12*\n\x04\x62\x61se\x18\x01 \x02(\x0b\x32\x1c.naothmessages.DoubleVector2\x12/\n\tdirection\x18\x02 \x02(\x0b\x32\x1c.naothmessages.DoubleVector2\x12\x0e\n\x06length\x18\x03 \x02(\x01*\x90\x01\n\x05\x43olor\x12\x08\n\x04none\x10\x00\x12\n\n\x06orange\x10\x01\x12\n\n\x06yellow\x10\x02\x12\x0b\n\x07skyblue\x10\x03\x12\t\n\x05white\x10\x04\x12\x07\n\x03red\x10\x05\x12\x08\n\x04\x62lue\x10\x06\x12\t\n\x05green\x10\x07\x12\t\n\x05\x62lack\x10\x08\x12\x08\n\x04pink\x10\t\x12\x08\n\x04gray\x10\n\x12\x10\n\x0cyellowOrange\x10\x0b*\x1f\n\x08\x43\x61meraID\x12\x07\n\x03top\x10\x00\x12\n\n\x06\x62ottom\x10\x01*\xa5\x03\n\x07JointID\x12\r\n\tHeadPitch\x10\x00\x12\x0b\n\x07HeadYaw\x10\x01\x12\x11\n\rRShoulderRoll\x10\x02\x12\x11\n\rLShoulderRoll\x10\x03\x12\x12\n\x0eRShoulderPitch\x10\x04\x12\x12\n\x0eLShoulderPitch\x10\x05\x12\x0e\n\nRElbowRoll\x10\x06\x12\x0e\n\nLElbowRoll\x10\x07\x12\r\n\tRElbowYaw\x10\x08\x12\r\n\tLElbowYaw\x10\t\x12\x10\n\x0cRHipYawPitch\x10\n\x12\x10\n\x0cLHipYawPitch\x10\x0b\x12\r\n\tRHipPitch\x10\x0c\x12\r\n\tLHipPitch\x10\r\x12\x0c\n\x08RHipRoll\x10\x0e\x12\x0c\n\x08LHipRoll\x10\x0f\x12\x0e\n\nRKneePitch\x10\x10\x12\x0e\n\nLKneePitch\x10\x11\x12\x0f\n\x0bRAnklePitch\x10\x12\x12\x0f\n\x0bLAnklePitch\x10\x13\x12\x0e\n\nRAnkleRoll\x10\x14\x12\x0e\n\nLAnkleRoll\x10\x15\x12\r\n\tLWristYaw\x10\x16\x12\r\n\tRWristYaw\x10\x17\x12\t\n\x05LHand\x10\x18\x12\t\n\x05RHand\x10\x19\x42\x1b\n\x19\x64\x65.naoth.rc.core.messages')
)

_COLOR = _descriptor.EnumDescriptor(
  name='Color',
  full_name='naothmessages.Color',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='none', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='orange', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='yellow', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='skyblue', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='white', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='red', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='blue', index=6, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='green', index=7, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='black', index=8, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='pink', index=9, number=9,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='gray', index=10, number=10,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='yellowOrange', index=11, number=11,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=539,
  serialized_end=683,
)
_sym_db.RegisterEnumDescriptor(_COLOR)

Color = enum_type_wrapper.EnumTypeWrapper(_COLOR)
_CAMERAID = _descriptor.EnumDescriptor(
  name='CameraID',
  full_name='naothmessages.CameraID',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='top', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='bottom', index=1, number=1,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=685,
  serialized_end=716,
)
_sym_db.RegisterEnumDescriptor(_CAMERAID)

CameraID = enum_type_wrapper.EnumTypeWrapper(_CAMERAID)
_JOINTID = _descriptor.EnumDescriptor(
  name='JointID',
  full_name='naothmessages.JointID',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='HeadPitch', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HeadYaw', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RShoulderRoll', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LShoulderRoll', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RShoulderPitch', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LShoulderPitch', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RElbowRoll', index=6, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LElbowRoll', index=7, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RElbowYaw', index=8, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LElbowYaw', index=9, number=9,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RHipYawPitch', index=10, number=10,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LHipYawPitch', index=11, number=11,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RHipPitch', index=12, number=12,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LHipPitch', index=13, number=13,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RHipRoll', index=14, number=14,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LHipRoll', index=15, number=15,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RKneePitch', index=16, number=16,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LKneePitch', index=17, number=17,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RAnklePitch', index=18, number=18,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LAnklePitch', index=19, number=19,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RAnkleRoll', index=20, number=20,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LAnkleRoll', index=21, number=21,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LWristYaw', index=22, number=22,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RWristYaw', index=23, number=23,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LHand', index=24, number=24,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RHand', index=25, number=25,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=719,
  serialized_end=1140,
)
_sym_db.RegisterEnumDescriptor(_JOINTID)

JointID = enum_type_wrapper.EnumTypeWrapper(_JOINTID)
none = 0
orange = 1
yellow = 2
skyblue = 3
white = 4
red = 5
blue = 6
green = 7
black = 8
pink = 9
gray = 10
yellowOrange = 11
top = 0
bottom = 1
HeadPitch = 0
HeadYaw = 1
RShoulderRoll = 2
LShoulderRoll = 3
RShoulderPitch = 4
LShoulderPitch = 5
RElbowRoll = 6
LElbowRoll = 7
RElbowYaw = 8
LElbowYaw = 9
RHipYawPitch = 10
LHipYawPitch = 11
RHipPitch = 12
LHipPitch = 13
RHipRoll = 14
LHipRoll = 15
RKneePitch = 16
LKneePitch = 17
RAnklePitch = 18
LAnklePitch = 19
RAnkleRoll = 20
LAnkleRoll = 21
LWristYaw = 22
RWristYaw = 23
LHand = 24
RHand = 25



_FLOATVECTOR2 = _descriptor.Descriptor(
  name='FloatVector2',
  full_name='naothmessages.FloatVector2',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='naothmessages.FloatVector2.x', index=0,
      number=1, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='y', full_name='naothmessages.FloatVector2.y', index=1,
      number=2, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=36,
  serialized_end=72,
)


_DOUBLEVECTOR2 = _descriptor.Descriptor(
  name='DoubleVector2',
  full_name='naothmessages.DoubleVector2',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='naothmessages.DoubleVector2.x', index=0,
      number=1, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='y', full_name='naothmessages.DoubleVector2.y', index=1,
      number=2, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=74,
  serialized_end=111,
)


_INTVECTOR2 = _descriptor.Descriptor(
  name='IntVector2',
  full_name='naothmessages.IntVector2',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='naothmessages.IntVector2.x', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='y', full_name='naothmessages.IntVector2.y', index=1,
      number=2, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=113,
  serialized_end=147,
)


_POSE2D = _descriptor.Descriptor(
  name='Pose2D',
  full_name='naothmessages.Pose2D',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='translation', full_name='naothmessages.Pose2D.translation', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rotation', full_name='naothmessages.Pose2D.rotation', index=1,
      number=2, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=149,
  serialized_end=226,
)


_DOUBLEVECTOR3 = _descriptor.Descriptor(
  name='DoubleVector3',
  full_name='naothmessages.DoubleVector3',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='naothmessages.DoubleVector3.x', index=0,
      number=1, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='y', full_name='naothmessages.DoubleVector3.y', index=1,
      number=2, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='z', full_name='naothmessages.DoubleVector3.z', index=2,
      number=3, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=228,
  serialized_end=276,
)


_DOUBLEVECTOR = _descriptor.Descriptor(
  name='DoubleVector',
  full_name='naothmessages.DoubleVector',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='v', full_name='naothmessages.DoubleVector.v', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=278,
  serialized_end=303,
)


_POSE3D = _descriptor.Descriptor(
  name='Pose3D',
  full_name='naothmessages.Pose3D',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='translation', full_name='naothmessages.Pose3D.translation', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rotation', full_name='naothmessages.Pose3D.rotation', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=305,
  serialized_end=412,
)


_LINESEGMENT = _descriptor.Descriptor(
  name='LineSegment',
  full_name='naothmessages.LineSegment',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='base', full_name='naothmessages.LineSegment.base', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='direction', full_name='naothmessages.LineSegment.direction', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='length', full_name='naothmessages.LineSegment.length', index=2,
      number=3, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=414,
  serialized_end=536,
)

_POSE2D.fields_by_name['translation'].message_type = _DOUBLEVECTOR2
_POSE3D.fields_by_name['translation'].message_type = _DOUBLEVECTOR3
_POSE3D.fields_by_name['rotation'].message_type = _DOUBLEVECTOR3
_LINESEGMENT.fields_by_name['base'].message_type = _DOUBLEVECTOR2
_LINESEGMENT.fields_by_name['direction'].message_type = _DOUBLEVECTOR2
DESCRIPTOR.message_types_by_name['FloatVector2'] = _FLOATVECTOR2
DESCRIPTOR.message_types_by_name['DoubleVector2'] = _DOUBLEVECTOR2
DESCRIPTOR.message_types_by_name['IntVector2'] = _INTVECTOR2
DESCRIPTOR.message_types_by_name['Pose2D'] = _POSE2D
DESCRIPTOR.message_types_by_name['DoubleVector3'] = _DOUBLEVECTOR3
DESCRIPTOR.message_types_by_name['DoubleVector'] = _DOUBLEVECTOR
DESCRIPTOR.message_types_by_name['Pose3D'] = _POSE3D
DESCRIPTOR.message_types_by_name['LineSegment'] = _LINESEGMENT
DESCRIPTOR.enum_types_by_name['Color'] = _COLOR
DESCRIPTOR.enum_types_by_name['CameraID'] = _CAMERAID
DESCRIPTOR.enum_types_by_name['JointID'] = _JOINTID
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FloatVector2 = _reflection.GeneratedProtocolMessageType('FloatVector2', (_message.Message,), dict(
  DESCRIPTOR = _FLOATVECTOR2,
  __module__ = 'CommonTypes_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.FloatVector2)
  ))
_sym_db.RegisterMessage(FloatVector2)

DoubleVector2 = _reflection.GeneratedProtocolMessageType('DoubleVector2', (_message.Message,), dict(
  DESCRIPTOR = _DOUBLEVECTOR2,
  __module__ = 'CommonTypes_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.DoubleVector2)
  ))
_sym_db.RegisterMessage(DoubleVector2)

IntVector2 = _reflection.GeneratedProtocolMessageType('IntVector2', (_message.Message,), dict(
  DESCRIPTOR = _INTVECTOR2,
  __module__ = 'CommonTypes_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.IntVector2)
  ))
_sym_db.RegisterMessage(IntVector2)

Pose2D = _reflection.GeneratedProtocolMessageType('Pose2D', (_message.Message,), dict(
  DESCRIPTOR = _POSE2D,
  __module__ = 'CommonTypes_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.Pose2D)
  ))
_sym_db.RegisterMessage(Pose2D)

DoubleVector3 = _reflection.GeneratedProtocolMessageType('DoubleVector3', (_message.Message,), dict(
  DESCRIPTOR = _DOUBLEVECTOR3,
  __module__ = 'CommonTypes_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.DoubleVector3)
  ))
_sym_db.RegisterMessage(DoubleVector3)

DoubleVector = _reflection.GeneratedProtocolMessageType('DoubleVector', (_message.Message,), dict(
  DESCRIPTOR = _DOUBLEVECTOR,
  __module__ = 'CommonTypes_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.DoubleVector)
  ))
_sym_db.RegisterMessage(DoubleVector)

Pose3D = _reflection.GeneratedProtocolMessageType('Pose3D', (_message.Message,), dict(
  DESCRIPTOR = _POSE3D,
  __module__ = 'CommonTypes_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.Pose3D)
  ))
_sym_db.RegisterMessage(Pose3D)

LineSegment = _reflection.GeneratedProtocolMessageType('LineSegment', (_message.Message,), dict(
  DESCRIPTOR = _LINESEGMENT,
  __module__ = 'CommonTypes_pb2'
  # @@protoc_insertion_point(class_scope:naothmessages.LineSegment)
  ))
_sym_db.RegisterMessage(LineSegment)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\031de.naoth.rc.core.messages'))
# @@protoc_insertion_point(module_scope)



Joints = {0: "HeadPitch", 1: "HeadYaw", 2: "RShoulderRoll", 3: "LShoulderRoll", 4: "RShoulderPitch",
          5: "LShoulderPitch", 6: "RElbowRoll", 7: "LElbowRoll", 8: "RElbowYaw", 9: "LElbowYaw", 10: "RHipYawPitch",
          11: "LHipYawPitch", 12: "RHipPitch", 13: "LHipPitch", 14: "RHipRoll", 15: "LHipRoll", 16: "RKneePitch",
          17: "LKneePitch", 18: "RAnklePitch", 19: "LAnklePitch", 20: "RAnkleRoll", 21: "LAnkleRoll"}


def get_joint_id(val):
    for key, value in Joints.items():
        if val == value:
            return key



import inspect as _inspect
import enum as _enum
import re as _re
import copy

from .. import pb as _pb
from ._experimental_xabsl import XABSLOption, XABSLAction


class Parser:
    """
    Parses raw log file frame member bytes using protobuf.
    If a name of a frame member does not match a class defined in protobuf,
    the corresponding class name needs to be defined using the register function.

    Ex: parser.register("ImageTop", "Image") where Image is the corresponding name of the protobuf class.
    """

    def __init__(self):
        self.protobuf_classes = {}

        # get all protobuf classes
        for _m in _inspect.getmembers(_pb, _inspect.ismodule):
            for _c in _inspect.getmembers(_m[1], _inspect.isclass):
                if not _c[0].startswith('_'):
                    self.protobuf_classes[_c[0]] = _c[1]

        self.name_to_type = {}

        # register representations that are not present in the protobuf messages
        self.register("ImageTop", "Image")
        self.register("CameraMatrixTop", "CameraMatrix")
        self.register("MotorJointData", "JointData")
        self.register("ScanLineEdgelPerceptTop", "ScanLineEdgelPercept")

    def register(self, name: str, _type: str):
        """
        Register name corresponding protobuf type.
        :param name: frame member name
        :param _type: class in protobuf
        """
        if _type not in self.protobuf_classes:
            raise ValueError(f'Class {_type} is not part of the protobuf defined types.')

        self.name_to_type[name] = _type

    def parse(self, name, data):
        """
        Parse frame member bytes using protobuf.
        :param name: of frame member
        :param data: payload bytes
        :returns member class
        """
        if name in self.name_to_type:
            name = self.name_to_type[name]
        # protobuf
        if name in self.protobuf_classes:
            message = self.protobuf_classes[name]()
            message.ParseFromString(data)
            return message
        else:
            raise ValueError(f'Class {name} is not part of the protobuf defined types.')


class SymbolParser:
    """
    Maps symbol ids to the corresponding name in BehaviorStateComplete
    """
    def __init__(self, symbol_list, enum_types):
        self.decimal_id_to_name = {decimal.id: decimal.name for decimal in symbol_list.decimal}
        self.boolean_id_to_name = {boolean.id: boolean.name for boolean in symbol_list.boolean}

        self.enum_id_to_type = {enum.id: enum_types[enum.typeId] for enum in symbol_list.enumerated}
        self.enum_id_to_name = {enum.id: enum.name for enum in symbol_list.enumerated}

    def parse_symbols(self, symbol_list):
        for decimal in symbol_list.decimal:
            yield self.decimal_id_to_name[decimal.id], decimal.value

        for boolean in symbol_list.boolean:
            yield self.boolean_id_to_name[boolean.id], boolean.value

        for enum in symbol_list.enumerated:
            enum_type = self.enum_id_to_type[enum.id]
            instance_name = self.enum_id_to_name[enum.id]
            
            # HACK: some values are set to invalid at the begining, such as numOfHeadMotion
            # those values are not represented in the xabsl enums. So we just set them to 
            # "unknown"
            try:
                value = enum_type(enum.value)
                #print(type(value))
            except:
                value = type('obj', (object,), {'name' : enum.value})
                print("use unkbnown for {} = {}".format(instance_name, enum.value))
            
            yield instance_name, value


class BehaviorFrame:
    def __init__(self):
        self.root_options = {}

        self.input_symbols = {}
        self.output_symbols = {}

    def __getitem__(self, option_name):
        return self.root_options[option_name]

    def __str__(self):
        return '\n'.join(XABSLOption.option_to_string(name, option) for name, option in self.root_options.items())


class BehaviorParser:
    def __init__(self, behavior_state=None):
        """
        :param behavior_state: BehaviorStateComplete member of the log file.
                               If not given, BehaviorParser must be initialized manually by calling initialize
        """
        self.agents = []

        # These fields are set in the initialize function
        self.enum_types = None
        self._input_symbol_parser = None
        self._output_symbol_parser = None
        self._options_complete = None

        self.initialized = False

        if behavior_state is not None:
            self.initialize(behavior_state)

    def initialize(self, behavior_state, force_initialization = False):
        """
        Behavior parser must be initialized with BehaviorStateComplete member of the log file
        (usually contained in the first log file frame)
        :param behavior_state: BehaviorStateComplete
        """
        # don't run twice
        if not force_initialization and self.initialized:
            return
        
        # parse agents
        self.agents = [agent for agent in behavior_state.agents]

        # initialize options
        self._options_complete = behavior_state.options

        # parse enum types
        self.enum_types = {
            typeId: self._parse_xabsl_enum(enumeration) for typeId, enumeration in enumerate(behavior_state.enumerations)
        }

        # initialize symbols
        self._input_symbol_parser = SymbolParser(behavior_state.inputSymbolList, self.enum_types)
        self._output_symbol_parser = SymbolParser(behavior_state.outputSymbolList, self.enum_types)

        # create a base behavior frame, all following frames are differential frames
        self.behavior_frame_complete = BehaviorFrame()
        # fill the base frame
        self.behavior_frame_complete = self.parse(behavior_state)
        
        self.initialized = True

    @staticmethod
    def _parse_xabsl_enum(xabsl_enum):
        """
        Creates python enum from protobuf xabsl_enum
        """
        def convert_enum_value(value):
            # remove enum name prefix from enum value
            element_name = value[len(xabsl_enum.name) + 1:]
            
            # replace _NAME_ values with NAME since they can't be used in pythons enum class
            match = _re.match(r'_(?P<NAME>\S*)_', element_name)
            if match:
                return match.group("NAME")

            return element_name
            
        enum_type = _enum.Enum(xabsl_enum.name, [(convert_enum_value(elem.name), elem.value) for elem in xabsl_enum.elements])
        
        '''
        # debug: print all values in the enum
        if xabsl_enum.name == "head.motion_type":
            print(xabsl_enum)
            print([(e.name, e.value) for e in enum_type])
            #quit()
        '''
        
        return enum_type

    def _parse_actions(self, option_sparse):
        if _pb.Messages_pb2.XABSLActionSparse.ActionType.Name(option_sparse.type) == 'Option':

            option_complete = self._options_complete[option_sparse.option.id]

            parsed_option = XABSLOption(option_sparse.option.timeOfExecution,
                                        option_complete.states[option_sparse.option.activeState],
                                        option_sparse.option.stateTime)

            decimals = iter(option_sparse.option.decimalParameters)
            booleans = iter(option_sparse.option.booleanParameters)
            enums = iter(option_sparse.option.enumeratedParameters)

            for parameter in option_complete.parameters:

                if _pb.Messages_pb2.XABSLSymbol.SymbolType.Name(parameter.type) == 'Decimal':
                    parsed_option.parameters[parameter.name] = next(decimals)
                elif _pb.Messages_pb2.XABSLSymbol.SymbolType.Name(parameter.type) == 'Boolean':
                    parsed_option.parameters[parameter.name] = next(booleans)
                elif _pb.Messages_pb2.XABSLSymbol.SymbolType.Name(parameter.type) == 'Enum':
                    type = self.enum_types[parameter.enumTypeId]
                    parsed_option.parameters[parameter.name] = type(next(enums))
                else:
                    raise NotImplementedError(f'Parameter type {parameter.type} not implemented')

            for sub_action in option_sparse.option.activeSubActions:
                action, name, parsed_sub_action = self._parse_actions(sub_action)
                if action == XABSLAction.XABSLOption:
                    parsed_option.sub_options[name] = parsed_sub_action
                elif action == XABSLAction.XABSLSymbol:
                    parsed_option.symbols[name] = parsed_sub_action
                else:
                    raise NotImplementedError(f'Action {action} not implemented.')

            return XABSLAction.XABSLOption, option_complete.name, parsed_option

        elif _pb.Messages_pb2.XABSLActionSparse.ActionType.Name(option_sparse.type) == 'SymbolAssignment':
            symbol = option_sparse.symbol

            if _pb.Messages_pb2.XABSLSymbol.SymbolType.Name(symbol.type) == 'Decimal':
                name = self._output_symbol_parser.decimal_id_to_name[symbol.id]
                value = symbol.decimalValue
            elif _pb.Messages_pb2.XABSLSymbol.SymbolType.Name(symbol.type) == 'Boolean':
                name = self._output_symbol_parser.boolean_id_to_name[symbol.id]
                value = symbol.boolValue
            elif _pb.Messages_pb2.XABSLSymbol.SymbolType.Name(symbol.type) == 'Enum':
                enum_type = self._output_symbol_parser.enum_id_to_type[symbol.id]
                name = enum_type.__name__
                value = enum_type(symbol.enumValue)
            else:
                raise NotImplementedError(f'XABSLSymbol type {_pb.Messages_pb2.XABSLSymbol.SymbolType.Name(symbol.type)} not implemented.')

            return XABSLAction.XABSLSymbol, name, value

        else:
            raise NotImplementedError(f'ActionType {_pb.Messages_pb2.XABSLActionSparse.ActionType.Name(option_sparse.type)} not implemented')

    def parse(self, behavior_state):
        """
        Parses the XABSL Behavior based on the BehaviorStateSparse member of a log file frame
        :param behavior_state: BehaviorStateSparse
        :returns parsed BehaviorFrame
        """
        if isinstance(behavior_state, _pb.Messages_pb2.BehaviorStateSparse) and not self.initialized:
            raise ValueError('BehaviorParser must be initialized with "BehaviorStateComplete" (call initialize).')

        # create a new frame as a copy of the base frame
        behavior_frame = copy.deepcopy(self.behavior_frame_complete)

        # parse input and output symbols
        for name, value in self._input_symbol_parser.parse_symbols(behavior_state.inputSymbolList):
            behavior_frame.input_symbols[name] = value
        for name, value in self._output_symbol_parser.parse_symbols(behavior_state.outputSymbolList):
            behavior_frame.output_symbols[name] = value

        # process active options
        if isinstance(behavior_state, _pb.Messages_pb2.BehaviorStateSparse):
            for option_sparse in behavior_state.activeRootActions:
                action, name, option = self._parse_actions(option_sparse)
                # roots must be options
                assert action == XABSLAction.XABSLOption
                behavior_frame.root_options[name] = option

        return behavior_frame



import os as _os
import io as _io
import struct as _struct
from ._parser import Parser


class Reader:
    class Frame:
        def __init__(self, reader, number):
            self.reader = reader
            self.number = number
            self.messages = {}

        def __getitem__(self, name):
            return self.getMessage(name)

        def getMessage(self, name):
            position, size, message = self.messages[name]

            if message is not None:
                return message

            self.reader.log.seek(position)
            data = self.reader.readBytes(size)

            message = self.reader.parser.parse(name, data)
            self.messages[name] = (position, size, message)
            return message

    def __init__(self, path, parser=Parser(), filter=lambda x: x):
        self.__open(path)
        self.log_file = self.log.name
        self.parser = parser
        self.filter = filter

        self.frames = []
        self.names = []

        self.scanPosition = 0
        # get the first frame
        self.__scanFrame()

    def __open(self, path):
        if isinstance(path, _io.IOBase):
            self.size = _os.stat(path.name).st_size
            if path.mode != "rb":
                raise Exception("File opened with the wrong mode! is {}, should be 'rb'".format(path.mode))
            self.log = path
        else:
            statinfo = _os.stat(path)
            self.size = statinfo.st_size
            self.log = open(path, "rb")

    def __getstate__(self):
        """ This is called before pickling. """
        state = self.__dict__.copy()
        del state['log']
        # NOTE: currently its not possible to preserve the filter
        del state['filter']
        return state

    def __setstate__(self, state):
        """ This is called while unpickling. """
        self.__dict__.update(state)
        self.__open(self.log_file)
        # NOTE: currently its not possible to restore the filter
        self.filter = lambda x: x

    def __scan(self):
        try:
            while True:
                self.__scanFrame()
                # todo: make it more efficient
                # if name not in self.names:
                #  self.names.append(name)

        except StopIteration as ex:
            print(ex)

    def __scanFrame(self):
        if self.scanPosition == -1:
            return False
        else:
            self.log.seek(self.scanPosition)

        # if self.idx > 30*100:
        #  return False

        last = (self.scanPosition * 100) / self.size

        try:
            currentFrame = None
            while True:
                # read message header
                frameNumber = self.readLong()

                if currentFrame is None:
                    currentFrame = Reader.Frame(self, frameNumber)

                if currentFrame.number != frameNumber:
                    self.log.seek(-4, 1)
                    self.scanPosition = self.log.tell()
                    self.frames.append(currentFrame)
                    break

                name = self.readString()
                message_size = self.readLong()

                currentFrame.messages[name] = (self.log.tell(), message_size, None)
                # skip message data
                self.log.seek(message_size, 1)

                # todo: make it more efficient
                if name not in self.names:
                    self.names.append(name)

            new = (self.scanPosition * 100) / self.size
            # if new > last:
            # print new

            # if new > 3:
            #  raise StopIteration

            return True

        except StopIteration as ex:
            self.scanPosition = -1
            return False

    def readBytes(self, bytes):
        if self.log is None:
            raise StopIteration

        if bytes == 0:
            return b''

        data = self.log.read(bytes)
        if len(data) != bytes:
            raise StopIteration

        return data

    def readLong(self):
        return _struct.unpack("=l", self.readBytes(4))[0]

    def readChar(self):
        return _struct.unpack("=c", self.readBytes(1))[0]

    # read a '\0' terminated string
    def readString(self):
        str = ''
        c = self.readChar().decode("utf-8")
        while c != "\0":
            str += c
            c = self.readChar().decode("utf-8")
        return str

    def __getitem__(self, i):
        while i + 1 >= len(self.frames):
            if not self.__scanFrame():
                break

        if i + 1 < len(self.frames):
            return self.filter(self.frames[i])
        else:
            raise StopIteration



#!/usr/bin/python
from setuptools import setup, find_packages
from pathlib import Path

# The directory containing this file
HERE = Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()
VERSION_STRING = (HERE / "VERSION").read_text()

setup(name='naoth',
      version=VERSION_STRING,
      author='NaoTH Berlin United',
      author_email='nao-team@informatik.hu-berlin.de',
      description='Python utils for the NaoTH toolchain',
      long_description=README,
      long_description_content_type="text/markdown",
      packages=find_packages(exclude=["tests"]),
      license='Apache License 2.0',
      zip_safe=False,
      setup_requires=['wheel'],
      install_requires=[
          'protobuf', 'numpy'
      ],
      python_requires='>=3.6.9',
      )




def load_data(type: str = None):
    from . import __get_file
    files = {
        'motion': ('walk_on_floor_motion.log', 'a7ba18cb744c05905879381a041a82a1'),
        'cognition': ('walk_on_floor_cognition.log', '58ea2fc18f3721e0c05cf0ba94cf055c')
    }

    if type:
        return __get_file(files[type][0], file_hash=files[type][1], cache_subdir='demo_motion')

    return {k: __get_file(v[0], file_hash=v[1], cache_subdir='demo_motion') for k, v in files.items()}



import numpy as _np


class Image:

    def __init__(self, proto, width=640, height=480, depth=3):
        #
        self._width = width
        self._height = height
        self._depth = depth
        
        self._yuv422 = _np.fromstring(proto, dtype=_np.uint8)
        self._yuv444 = None
        self._rgb = None

    @property
    def yuv422(self):
        """Get the image as YUV422 numpy array"""
        return self._yuv422

    @property
    def yuv444(self):
        """Get the image as YUV444 numpy array"""
        if self._yuv444 is None:
            self._yuv422_to_yuv444()
        return self._yuv444

    def _yuv422_to_yuv444(self):
        # read each channel of yuv422 separately
        y = self._yuv422[0::2]
        u = self._yuv422[1::4]
        v = self._yuv422[3::4]

        # convert from yuv422 to yuv444
        self._yuv444 = _np.zeros(self._width * self._height * self._depth, dtype=_np.uint8)
        self._yuv444[0::3] = y
        self._yuv444[1::6] = u
        self._yuv444[2::6] = v
        self._yuv444[4::6] = u
        self._yuv444[5::6] = v

    @property
    def rgb(self):
        """Get the image as rgb numpy array."""
        if self._rgb is None:
            self._yuv444_to_rgb()

        return self._rgb

    def _yuv444_to_rgb(self):
        """Converts yuv444 image to rgb888"""
        m = _np.array([
            [ 1.0,                      1.0,                1.0                   ],
            [-0.000007154783816076815, -0.3441331386566162, 1.7720025777816772    ],
            [ 1.4019975662231445,      -0.7141380310058594, 0.00001542569043522235]
        ])

        rgb = _np.dot(self.yuv444.reshape((self._height, self._width, self._depth)), m)
        rgb[:, :, 0] -= 179.45477266423404
        rgb[:, :, 1] += 135.45870971679688
        rgb[:, :, 2] -= 226.8183044444304

        self._rgb = rgb.clip(0, 255).astype(_np.uint8)

    @property
    def brightness(self):
        """Get only the brightness channel (luma, luminance) of the image as numpy array."""
        y = self._yuv422[0::2]
        return y.reshape((self._height, self._width))

    @property
    def luma(self):
        """Get only the brightness channel (luma, luminance) of the image as numpy array."""
        return self.brightness

    @property
    def chroma_b(self):
        """Get only the chrominance blue channel (blue projection/difference) of the image as numpy array."""
        b = self.yuv444[1::3]
        return b.reshape((self._height, self._width))

    @property
    def chroma_r(self):
        """Get only the chrominance red channel (red projection/difference) of the image as numpy array."""
        r = self.yuv444[2::3]
        return r.reshape((self._height, self._width))



import numpy as _np


class Camera:
    """
    Represents the camera info of a nao with additional methods for the geometry.
    """

    def __init__(self, width=640, height=480, opening_angle_diagonal=_np.radians(72.6)):
        self._width = width
        self._height = height
        self._opening_angle_diagonal = opening_angle_diagonal
        self._optical_center = (0, 0)
        self._focal_length = 0

        self._update_vars()

    def _update_vars(self):
        """Calculates the focal length and the optical center of this camera.
        It is sufficient to calculate those values once or only if one of the other values changes.
        """
        self._focal_length = (0.5 * _np.sqrt(self._width ** 2 + self._height ** 2)) \
                             / _np.tan(0.5 * self._opening_angle_diagonal)

        self._optical_center = self._width / 2, self._height / 2

    @property
    def width(self):
        """Returns the width of this camera image."""
        return self._width

    @width.setter
    def width(self, new_width):
        """Sets the new width of the camera image."""
        self._width = new_width
        self._update_vars()

    @property
    def height(self):
        """Returns the height of this camera image."""
        return self._height

    @height.setter
    def height(self, new_height):
        """Sets the new height of the camera image."""
        self._height = new_height
        self._update_vars()

    @property
    def opening_angle_diagonal(self):
        """Returns the opening angle diagonal of this camera."""
        return self._opening_angle_diagonal

    @opening_angle_diagonal.setter
    def opening_angle_diagonal(self, new_opening_angle_diagonal):
        """Sets the new opening angle diagonal of this camera"""
        self._opening_angle_diagonal = new_opening_angle_diagonal
        self._update_vars()

    def focal_length(self):
        """Returns the focal length of the camera."""
        return self._focal_length

    def optical_center(self):
        """Returns the optical center of the camera image as tuple (half width, half height)."""
        return self._optical_center

    def calculate_artificial_horizon(self, c_rot):
        """
        Calculate the vanishing line of the camera in image coordinates
        (below this line pixels can be projected onto the ground)
        :param c_rot: camera rotation matrix
        :return: Start and end point tuple defining the vanishing line in image coordinates
        """
        r31 = c_rot[2, 0]
        r32 = c_rot[2, 1]
        r33 = c_rot[2, 2]

        if r33 == 0:
            r33 = 1e-5

        x1 = 0
        x2 = self._width - 1

        v1 = self._focal_length
        v2 = self._width / 2
        v3 = self._height / 2

        y1 = v3 + (r31 * v1 + r32 * v2) / r33
        y2 = v3 + (r31 * v1 - r32 * v2) / r33

        # Mirror ends of horizon if Camera rotated to the left
        if (c_rot @ _np.array([0, 0, 1]))[2] < 0:
            x1, x2 = x2, x1
            y1, y2 = y2, y1

        return _np.array((x1, y1)), _np.array((x2, y2))

    def project(self, x, y, c_rot, c_trans):
        """
        Project an image point onto the ground plane using the camera matrix
        :param x in image
        :param y in image
        :param c_rot: 3x3 camera rotation numpy array
        :param c_trans: 3d camera translation numpy array
        :return: point on the ground plane as 2d numpy array
        """

        half_x, half_y = self._optical_center

        pixel_v = _np.array((
            self._focal_length,
            half_x - x,
            half_y - y
        ))

        v_rot = c_rot @ pixel_v

        field_v = v_rot[:2] * (c_trans[2] / (-v_rot[2]))
        field_v += c_trans[:2]

        return field_v

    def relative_point_to_image(self, point, c_rot, c_trans):
        if len(point) == 2:
            point = _np.array((*point, 0))
        elif len(point) != 3:
            raise ValueError('Given point must have a dimension of 2 of 3')

        epsilon = 1e-13

        # vector: O --> point (in camera coordinates)
        vector_to_point = _np.linalg.inv(c_rot) @ (point - c_trans)

        # the point is behind the camera plane
        if vector_to_point[0] <= epsilon:
            return None

        factor = self.focal_length() / vector_to_point[0]
        center_x, center_y = self._optical_center

        point_in_image = _np.array((
                - (vector_to_point[1] * factor) + 0.5 + center_x
                - (vector_to_point[2] * factor) + 0.5 + center_y
        ))

        return point_in_image

    def __str__(self):
        return 'CAMERA INFO\n' \
               'width={}\n' \
               'height={}\n' \
               'opening_angle_diagonal={}'.format(self.width, self.height, self.opening_angle_diagonal)

class LogCamera(Camera):
    """
    CameraInfo parsed from a log file frame
    """

    def __init__(self, camera_info_data):
        Camera.__init__(self, camera_info_data.resolutionWidth,
                            camera_info_data.resolutionHeight,
                            camera_info_data.openingAngleDiagonal)

        self.data = camera_info_data

        # self.pixel_size = camera_info_data.pixelSize
        # self.focus = camera_info_data.focus

    def __getattr__(self, attr):
        return getattr(self.data, attr)

    def __str__(self):
        return 'CAMERA INFO\n' \
               'width={}\n' \
               'height={}\n' \
               'opening_angle_diagonal={}\n' \
               'pixelSize={}\n' \
               'focus={}'.format(self.width, self.height, self.opening_angle_diagonal, self.pixelSize, self.focus)
